@@ -11,12 +11,10 @@
 /*                                                                       */
 /*************************************************************************/
 
-#ifdef USE_ANYSPLIT
-#include "anysplit.h"
-#endif
 #include "dict-api.h"
 #include "dict-common.h"
 #include "externs.h"
+#include "pp_knowledge.h"
 #include "regex-morph.h"
 #include "spellcheck.h"
 #include "string-set.h"
@@ -27,6 +25,49 @@
 #include "dict-file/read-dict.h"
 #include "dict-file/word-file.h"
 
+#ifdef USE_ANYSPLIT
+#include "anysplit.h"
+#endif
+
+/* ======================================================================== */
+/* Affix type finding */
+
+/**
+ * Return TRUE if the word is a suffix.
+ *
+ * Suffixes have the form =asdf.asdf or possibly just =asdf without
+ * the dot (subscript mark). The "null" suffixes have the form
+ * =.asdf (always with the ubscript mark, as there are several).
+ * Ordinary equals signs appearing in regular text are either = or =[!].
+ */
+bool is_suffix(const char infix_mark, const char* w)
+{
+	if (infix_mark != w[0]) return false;
+	if (1 >= strlen(w)) return false;
+	if (0 == strncmp("[!", w+1, 2)) return false;
+#if SUBSCRIPT_MARK == '.'
+	/* Hmmm ... equals signs look like suffixes, but they are not ... */
+	if (0 == strcmp("=.v", w)) return false;
+	if (0 == strcmp("=.eq", w)) return false;
+#endif
+	return true;
+}
+
+/**
+ * Return TRUE if the word seems to be in stem form.
+ * Stems are signified by including = sign which is preceded by the subscript
+ * mark.  Examples (. represented the subscript mark): word.= word.=[!]
+ */
+bool is_stem(const char* w)
+{
+	const char *subscrmark = strchr(w, SUBSCRIPT_MARK);
+
+	if (NULL == subscrmark) return false;
+	if (subscrmark == w) return false;
+	if (STEM_MARK != subscrmark[1]) return false;
+	return true;
+}
+
 /* ======================================================================== */
 /* Replace the right-most dot with SUBSCRIPT_MARK */
 void patch_subscript(char * s)
@@ -102,17 +143,17 @@ const char * dictionary_get_lang(Dictionary dict)
  *
  * The returned list must be freed with free_lookup_list().
  */
-Dict_node * dictionary_lookup_list(Dictionary dict, const char *s)
+Dict_node * dictionary_lookup_list(const Dictionary dict, const char *s)
 {
 	return dict->lookup_list(dict, s);
 }
 
-void free_lookup_list(Dictionary dict, Dict_node *llist)
+void free_lookup_list(const Dictionary dict, Dict_node *llist)
 {
 	dict->free_lookup(dict, llist);
 }
 
-bool boolean_dictionary_lookup(Dictionary dict, const char *s)
+bool boolean_dictionary_lookup(const Dictionary dict, const char *s)
 {
 	return dict->lookup(dict, s);
 }
@@ -121,7 +162,7 @@ bool boolean_dictionary_lookup(Dictionary dict, const char *s)
  * Return true if word is in dictionary, or if word is matched by
  * regex.
  */
-bool find_word_in_dict(Dictionary dict, const char * word)
+bool find_word_in_dict(const Dictionary dict, const char * word)
 {
 	const char * regex_name;
 	if (boolean_dictionary_lookup (dict, word)) return true;
@@ -248,10 +289,10 @@ static inline void free_dict_node(Dict_node *dn)
 	xfree((char *)dn, sizeof(Dict_node));
 }
 
-
-static void free_Exp_list(Exp * e)
+void free_Exp_list(Exp_list * eli)
 {
 	Exp * e1;
+	Exp * e = eli->exp_list;
 	for (; e != NULL; e = e1)
 	{
 		e1 = e->next;
@@ -275,16 +316,16 @@ static void free_dictionary(Dictionary dict)
 {
 	free_dict_node_recursive(dict->root);
 	free_Word_file(dict->word_file_header);
-	free_Exp_list(dict->exp_list);
+	free_Exp_list(&dict->exp_list);
 }
 
-
 static void affix_list_delete(Dictionary dict)
 {
+	int i;
 	Afdict_class * atc;
-	for (atc = dict->afdict_class; 0 != atc->length; atc++)
+	for (i=0, atc = dict->afdict_class; i < AFDICT_NUM_ENTRIES; i++, atc++)
 	{
-		xfree((void *)atc->string, atc->mem_elems);
+		if (atc->string) free(atc->string);
 	}
 	free(dict->afdict_class);
 }
@@ -307,15 +348,12 @@ void dictionary_delete(Dictionary dict)
 	}
 	spellcheck_destroy(dict->spell_checker);
 
-#ifdef USE_FAT_LINKAGES
-	connector_set_delete(dict->andable_connector_set);
-#endif /* USE_FAT_LINKAGES */
 	connector_set_delete(dict->unlimited_connector_set);
 
 	if (dict->close) dict->close(dict);
 
-	post_process_close(dict->postprocessor);
-	post_process_close(dict->constituent_pp);
+	pp_knowledge_close(dict->base_knowledge);
+	pp_knowledge_close(dict->hpsg_knowledge);
 	string_set_delete(dict->string_set);
 	free_regexs(dict->regex_root);
 #ifdef USE_ANYSPLIT
@@ -374,7 +412,7 @@ void print_expression(Exp * n)
 /**
  * print the expression, in infix-style
  */
-static void print_expression_parens(Exp * n, int need_parens)
+static void print_expression_parens(const Exp * n, int need_parens)
 {
 	E_list * el;
 	int i, icost;
@@ -457,7 +495,7 @@ static void print_expression_parens(Exp * n, int need_parens)
 	if ((icost == 0) && need_parens) printf(")");
 }
 
-void print_expression(Exp * n)
+void print_expression(const Exp * n)
 {
 	print_expression_parens(n, false);
 	printf("\n");
